@@ -16,7 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  *
- * $Id: compiler.h,v 1.15 2005/01/15 14:24:28 ehofman Exp $
+ * $Id: compiler.h,v 1.21 2005/10/06 11:06:27 ehofman Exp $
  *
  **************************************************************************/
 
@@ -324,7 +324,7 @@
 #endif // Native SGI compilers
 
 
-#if defined ( sun )
+#if defined (__sun)
 #  include <strings.h>
 #  include <memory.h>
 #  if defined ( __cplusplus )
@@ -372,11 +372,15 @@
 
 #ifdef __APPLE__
 #  define SG_GL_H <OpenGL/gl.h>
+#  define SG_GLX_H <AGL/agl.h>
 #  define SG_GLU_H <OpenGL/glu.h>
 #  define SG_GLEXT_H <OpenGL/glext.h>
 #  define SG_GLUT_H <GLUT/glut.h>
+
+inline int (isnan)(double r) { return !(r <= 0 || r >= 0); }
 #else
 #  define SG_GL_H <GL/gl.h>
+#  define SG_GLX_H <GL/glx.h>
 #  define SG_GLU_H <GL/glu.h>
 #  define SG_GLEXT_H <GL/glext.h>
 #  define SG_GLUT_H <GL/glut.h>
@@ -464,3 +468,4 @@ inline const_mem_fun_ref_t<_Ret,_Tp> mem_fun_ref(_Ret (_Tp::*__f)() const)
 #endif // SG_INCOMPLETE_FUNCTIONAL
 
 #endif // _SG_COMPILER_H
+
