@@ -20,7 +20,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  *
- * $Id: moonpos.cxx,v 1.1.1.1 2002/09/07 02:58:20 curt Exp $
+ * $Id: moonpos.cxx,v 1.2 2005/10/25 13:48:58 ehofman Exp $
  **************************************************************************/
 
 
@@ -172,7 +172,7 @@ void MoonPos::updatePosition(double mjd, double lst, double lat, Star *ourSun)
   // SG_LOG( SG_GENERAL, SG_INFO, "rho = " << rho );
   
   if (geoRa < 0)
-    geoRa += (2*SGD_PI);
+    geoRa += SGD_2PI;
   
   HA = lst - (3.8197186 * geoRa);
   /* SG_LOG( SG_GENERAL, SG_INFO, "t->getLst() = " << t->getLst() 
