@@ -20,7 +20,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  *
- * $Id: star.cxx,v 1.1.1.1 2002/09/07 02:58:20 curt Exp $
+ * $Id: star.cxx,v 1.2 2005/11/11 13:19:58 ehofman Exp $
  **************************************************************************/
 
 
@@ -80,7 +80,7 @@ void Star::updatePosition(double mjd)
   double 
     actTime, eccAnom, 
     xv, yv, v, r,
-    xe, ye, ze, ecl;
+    xe, ecl;
 
   updateOrbElements(mjd);
   
