@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: sg_binobj.cxx,v 1.6 2004/11/19 21:44:16 curt Exp $
+// $Id: sg_binobj.cxx,v 1.9 2005/10/12 16:43:26 curt Exp $
 //
 
 
@@ -240,8 +240,8 @@ static void read_object( gzFile fp,
 	if ( nbytes > buf.get_size() ) { buf.resize( nbytes ); }
 	char *ptr = buf.get_ptr();
 	sgReadBytes( fp, nbytes, ptr );
-	int count = nbytes / (idx_size * sizeof(short));
-	short *sptr = (short *)ptr;
+	int count = nbytes / (idx_size * sizeof(unsigned short));
+	unsigned short *sptr = (unsigned short *)ptr;
 	int_list vs; vs.clear();
 	int_list ns; ns.clear();
 	int_list cs; cs.clear();
@@ -249,7 +249,7 @@ static void read_object( gzFile fp,
 	for ( k = 0; k < count; ++k ) {
 	    if ( sgIsBigEndian() ) {
 		for ( idx = 0; idx < idx_size; ++idx ) {
-		    sgEndianSwap( (unsigned short *)&(sptr[idx]) );
+		    sgEndianSwap( (uint16_t *)&(sptr[idx]) );
 		}
 	    }
 	    idx = 0;
@@ -403,9 +403,9 @@ bool SGBinObject::read_bin( const string& file ) {
 
 		double *dptr = (double *)ptr;
 		if ( sgIsBigEndian() ) {
-		    sgEndianSwap( (uint64 *)&(dptr[0]) );
-		    sgEndianSwap( (uint64 *)&(dptr[1]) );
-		    sgEndianSwap( (uint64 *)&(dptr[2]) );
+		    sgEndianSwap( (uint64_t *)&(dptr[0]) );
+		    sgEndianSwap( (uint64_t *)&(dptr[1]) );
+		    sgEndianSwap( (uint64_t *)&(dptr[2]) );
 		}
 		gbs_center = Point3D( dptr[0], dptr[1], dptr[2] );
 		// cout << "Center = " << gbs_center << endl;
@@ -413,7 +413,7 @@ bool SGBinObject::read_bin( const string& file ) {
 		
 		float *fptr = (float *)ptr;
 		if ( sgIsBigEndian() ) {
-		    sgEndianSwap( (unsigned int *)fptr );
+		    sgEndianSwap( (uint32_t *)fptr );
 		}
 		gbs_radius = fptr[0];
 		// cout << "Bounding radius = " << gbs_radius << endl;
@@ -443,9 +443,9 @@ bool SGBinObject::read_bin( const string& file ) {
 		wgs84_nodes.reserve( count );
 		for ( k = 0; k < count; ++k ) {
 		    if ( sgIsBigEndian() ) {
-			sgEndianSwap( (unsigned int *)&(fptr[0]) );
-			sgEndianSwap( (unsigned int *)&(fptr[1]) );
-			sgEndianSwap( (unsigned int *)&(fptr[2]) );
+			sgEndianSwap( (uint32_t *)&(fptr[0]) );
+			sgEndianSwap( (uint32_t *)&(fptr[1]) );
+			sgEndianSwap( (uint32_t *)&(fptr[2]) );
 		    }
 		    wgs84_nodes.push_back( Point3D(fptr[0], fptr[1], fptr[2]) );
 		    fptr += 3;
@@ -476,10 +476,10 @@ bool SGBinObject::read_bin( const string& file ) {
 		colors.reserve(count);
 		for ( k = 0; k < count; ++k ) {
 		    if ( sgIsBigEndian() ) {
-			sgEndianSwap( (unsigned int *)&(fptr[0]) );
-			sgEndianSwap( (unsigned int *)&(fptr[1]) );
-			sgEndianSwap( (unsigned int *)&(fptr[2]) );
-			sgEndianSwap( (unsigned int *)&(fptr[3]) );
+			sgEndianSwap( (uint32_t *)&(fptr[0]) );
+			sgEndianSwap( (uint32_t *)&(fptr[1]) );
+			sgEndianSwap( (uint32_t *)&(fptr[2]) );
+			sgEndianSwap( (uint32_t *)&(fptr[3]) );
 		    }
 		    colors.push_back( Point3D( fptr[0], fptr[1], fptr[2] ) );
 		    fptr += 4;
@@ -544,8 +544,8 @@ bool SGBinObject::read_bin( const string& file ) {
 		texcoords.reserve(count);
 		for ( k = 0; k < count; ++k ) {
 		    if ( sgIsBigEndian() ) {
-			sgEndianSwap( (unsigned int *)&(fptr[0]) );
-			sgEndianSwap( (unsigned int *)&(fptr[1]) );
+			sgEndianSwap( (uint32_t *)&(fptr[0]) );
+			sgEndianSwap( (uint32_t *)&(fptr[1]) );
 		    }
 		    texcoords.push_back( Point3D( fptr[0], fptr[1], 0 ) );
 		    fptr += 2;
@@ -656,7 +656,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     // write header magic
     sgWriteUInt( fp, SG_FILE_MAGIC_NUMBER );
     time_t calendar_time = time(NULL);
-    sgWriteLong( fp, (long int)calendar_time );
+    sgWriteLong( fp, (int32_t)calendar_time );
 
     // calculate and write number of top level objects
     string material;
