@@ -22,7 +22,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sg_random.h,v 1.2 2004/11/19 21:47:05 curt Exp $
+// $Id: sg_random.h,v 1.3 2005/04/30 09:59:12 ehofman Exp $
 
 
 #ifndef _SG_RANDOM_H
@@ -40,6 +40,13 @@ extern "C" {
  */
 void sg_srandom_time();
 
+/**
+ * Seed the random number generater with time() in 10 minute intervals
+ * so we get the same sequence within 10 minutes interval.
+ * This is useful for synchronizing two display systems.
+ */
+void sg_srandom_time_10();
+
 /**
  * Seed the random number generater with your own seed so can set up
  * repeatable randomization.
