@@ -214,7 +214,10 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
       }
     }
 
-    push_state(node, atts.getValue("type"), mode);
+    const char *type = atts.getValue("type");
+    if (type)
+      node->clearValue();
+    push_state(node, type, mode);
   }
 }
 
@@ -588,6 +591,9 @@ copyProperties (const SGPropertyNode *in, SGPropertyNode *out)
     }
   }
 
+  				// copy the attributes.
+  out->setAttributes( in->getAttributes() );
+
 				// Next, copy the children.
   int nChildren = in->nChildren();
   for (int i = 0; i < nChildren; i++) {
