@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: matmodel.cxx,v 1.5 2004/11/19 21:44:17 curt Exp $
+// $Id: matmodel.cxx,v 1.6 2005/10/27 08:21:58 ehofman Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -110,7 +110,7 @@ SGMatModel::~SGMatModel ()
 {
   for (unsigned int i = 0; i < _models.size(); i++) {
     if (_models[i] != 0) {
-      _models[i]->deRef();
+      ssgDeRefDelete(_models[i]);
       _models[i] = 0;
     }
   }
