@@ -13,7 +13,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: cloud.cxx,v 1.39 2005/01/11 16:02:39 curt Exp $
+// $Id: cloud.cxx,v 1.43 2005/09/05 08:17:37 ehofman Exp $
 
 
 #include <simgear/compiler.h>
@@ -21,13 +21,6 @@
 // #include <stdio.h>
 #include <math.h>
 
-#if defined (__APPLE__)
-// any C++ header file undefines isinf and isnan
-// so this should be included before <iostream>
-inline int (isinf)(double r) { return isinf(r); }
-inline int (isnan)(double r) { return isnan(r); } 
-#endif
-
 #include <plib/sg.h>
 #include <plib/ssg.h>
 
@@ -39,6 +32,8 @@ inline int (isnan)(double r) { return isnan(r); }
 #include <simgear/screen/extensions.hxx>
 #include <simgear/screen/texture.hxx>
 
+#include "newcloud.hxx"
+#include "cloudfield.hxx"
 #include "cloud.hxx"
 
 #if defined(__MINGW32__)
@@ -57,7 +52,7 @@ inline int (isnan)(double r) { return isnan(r); }
 static ssgStateSelector *layer_states[SGCloudLayer::SG_MAX_CLOUD_COVERAGES];
 static bool state_initialized = false;
 static bool bump_mapping = false;
-static int nb_texture_unit = 0;
+static GLint nb_texture_unit = 0;
 static ssgTexture *normal_map[SGCloudLayer::SG_MAX_CLOUD_COVERAGES][2] = { 0 };
 static ssgTexture *color_map[SGCloudLayer::SG_MAX_CLOUD_COVERAGES][2] = { 0 };
 static GLuint normalization_cube_map;
@@ -221,12 +216,14 @@ SGCloudLayer::SGCloudLayer( const string &tex_path ) :
     layer[0] = layer[1] = layer[2] = layer[3] = NULL;
 
     layer_root->addKid(layer_transform);
+	layer3D = new SGCloudField;
     rebuild();
 }
 
 // Destructor
 SGCloudLayer::~SGCloudLayer()
 {
+	delete layer3D;
     delete vertices;
     delete indices;
     delete layer_root;		// deletes layer_transform and layer as well
@@ -460,6 +457,8 @@ SGCloudLayer::rebuild()
 
             layer_states[SG_CLOUD_CLEAR] = 0;
         }
+		SGNewCloud::loadTextures(texture_path.str());
+		layer3D->buildTestLayer();
     }
 
     if ( bump_mapping ) {
@@ -837,6 +836,7 @@ bool SGCloudLayer::reposition( sgVec3 p, sgVec3 up, double lon, double lat,
         last_lat = lat;
     }
 
+	layer3D->reposition( p, up, lon, lat, alt, dt, direction, speed);
     return true;
 }
 
@@ -844,6 +844,9 @@ bool SGCloudLayer::reposition( sgVec3 p, sgVec3 up, double lon, double lat,
 void SGCloudLayer::draw( bool top ) {
     if ( layer_coverage != SG_CLOUD_CLEAR ) {
 
+		if ( SGCloudField::enable3D && layer3D->is3D())
+			layer3D->Render();
+		else
         if ( bump_mapping && enable_bump_mapping ) {
 
             sgMat4 modelview,
