@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sky.cxx,v 1.22 2005/01/11 16:02:39 curt Exp $
+// $Id: sky.cxx,v 1.24 2005/10/25 13:48:58 ehofman Exp $
 
 
 #include <plib/sg.h>
@@ -29,7 +29,7 @@
 #include <simgear/math/sg_random.h>
 
 #include "sky.hxx"
-
+#include "cloudfield.hxx"
 
 // Constructor
 SGSky::SGSky( void ) {
@@ -187,6 +187,8 @@ void SGSky::preDraw( float alt, float fog_exp2_density ) {
             // in cloud layer
 
             // bail now and don't draw any clouds
+			if( cloud_layers[i]->get_layer3D()->is3D() && SGCloudField::enable3D )
+				continue;
             in_cloud = i;
         } else {
             // above cloud layer
@@ -286,7 +288,8 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 	    ratio = 1.0;
 	}
 
-        if ( cloud_layers[i]->getCoverage() == SGCloudLayer::SG_CLOUD_CLEAR ) {
+        if ( cloud_layers[i]->getCoverage() == SGCloudLayer::SG_CLOUD_CLEAR ||
+			cloud_layers[i]->get_layer3D()->is3D() && SGCloudField::enable3D) {
             // do nothing, clear layers aren't drawn, don't affect
             // visibility andn dont' need to be faded in or out.
         } else if ( (cloud_layers[i]->getCoverage() == 
@@ -331,13 +334,13 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 		// modify actual_visibility based on puff envelope
 
 		if ( puff_progression <= ramp_up ) {
-		    double x = 0.5 * SGD_PI * puff_progression / ramp_up;
+		    double x = SGD_PI_2 * puff_progression / ramp_up;
 		    double factor = 1.0 - sin( x );
 		    // cout << "ramp up = " << puff_progression
 		    //      << "  factor = " << factor << endl;
 		    effvis = effvis * factor;
 		} else if ( puff_progression >= ramp_up + puff_length ) {
-		    double x = 0.5 * SGD_PI * 
+		    double x = SGD_PI_2 * 
 			(puff_progression - (ramp_up + puff_length)) /
 			ramp_down;
 		    double factor = sin( x );
