@@ -1,7 +1,12 @@
-/* $Id: tr.h,v 1.1.1.1 2002/09/07 02:58:19 curt Exp $ */
+/* $Id: tr.h,v 1.2 2004/11/18 19:10:34 curt Exp $ */
 
 /*
  * $Log: tr.h,v $
+ * Revision 1.2  2004/11/18 19:10:34  curt
+ * Abstract out location of gl.h, glut.h, and glu.h includes so that we can
+ * make the Mac platform happy since they put these in a different place compared
+ * to the rest of the world.
+ *
  * Revision 1.1.1.1  2002/09/07 02:58:19  curt
  * Initial revsion of Simgear-0.3.0
  *
@@ -75,8 +80,9 @@
 #ifndef TR_H
 #define TR_H
 
+#include <simgear/compiler.h>
 
-#include <GL/gl.h>
+#include SG_GL_H
 
 
 //#ifdef __cplusplus
