@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: xmlsound.cxx,v 1.11 2004/11/21 17:05:42 ehofman Exp $
+// $Id: xmlsound.cxx,v 1.13 2005/11/12 10:26:21 ehofman Exp $
 
 #include <simgear/compiler.h>
 
@@ -262,15 +262,20 @@ SGXmlSound::init(SGPropertyNode *root, SGPropertyNode *node, SGSoundMgr *sndmgr,
       outer_gain = pos->getDoubleValue("outer-gain", 0.0);
    }
    
-
    //
    // Initialize the sample
    //
    _mgr = sndmgr;
    if ( (_sample = _mgr->find(_name)) == NULL ) {
-       _sample = new SGSoundSample( path.c_str(),
-                                    node->getStringValue("path", ""),
-                                    true );
+       // FIXME: Does it make sense to overwrite a previous entry's
+       // configuration just because a new entry has the same name?
+       // Note that we can't match on identical "path" because we the
+       // new entry could be at a different location with different
+       // configuration so we need a new sample which creates a new
+       // "alSource".  The semantics of what is going on here seems
+       // confused and needs to be thought through more carefully.
+        _sample = new SGSoundSample( path.c_str(),
+                                    node->getStringValue("path", "") );
 
        _mgr->add( _sample, _name );
    }
