@@ -19,7 +19,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sg_time.cxx,v 1.6 2004/11/19 21:47:05 curt Exp $
+// $Id: sg_time.cxx,v 1.8 2005/10/25 13:48:59 ehofman Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -279,7 +279,7 @@ void SGTime::updateLocal( double lon_rad, double lat_rad, const string& root ) {
         // not within -180 ... 180
         lon_rad = 0.0;
     }
-    if ( lat_rad < -SGD_PI * 0.5 || lat_rad > SGD_PI * 0.5 ) {
+    if ( lat_rad < -SGD_PI_2 || lat_rad > SGD_PI_2 ) {
         // not within -90 ... 90
         lat_rad = 0.0;
     }
@@ -530,7 +530,7 @@ time_t sgTimeGetGMT(int year, int month, int day, int hour, int min, int sec)
 #else // ! defined ( MK_TIME_IS_GMT )
 
     // timezone seems to work as a proper offset for Linux & Solaris
-#   if defined( __linux__ ) || defined( __sun__ ) ||defined(__CYGWIN__)
+#   if defined( __linux__ ) || defined(__sun) ||defined(__CYGWIN__)
 #       define TIMEZONE_OFFSET_WORKS 1
 #   endif
 
