@@ -8,6 +8,7 @@
 #include "xmlparse.h"
 
 #include STL_FSTREAM
+#include STL_IOSTREAM
 
 SG_USING_STD(ifstream);
 
@@ -219,34 +220,37 @@ readXML (istream &input, XMLVisitor &visitor, const string &path)
 
 				// FIXME: get proper error string from system
     if (!input.good()) {
-      XML_ParserFree(parser);
-      throw sg_io_exception("Problem reading file",
+      sg_io_exception ex ("Problem reading file",
 			    sg_location(path,
 					XML_GetCurrentLineNumber(parser),
 					XML_GetCurrentColumnNumber(parser)),
 			    "SimGear XML Parser");
+      XML_ParserFree(parser);
+      throw ex;
     }
 
     input.read(buf,16384);
     if (!XML_Parse(parser, buf, input.gcount(), false)) {
-      XML_ParserFree(parser);
-      throw sg_io_exception(XML_ErrorString(XML_GetErrorCode(parser)),
+      sg_io_exception ex (XML_ErrorString(XML_GetErrorCode(parser)),
 			    sg_location(path,
 					XML_GetCurrentLineNumber(parser),
 					XML_GetCurrentColumnNumber(parser)),
 			    "SimGear XML Parser");
+      XML_ParserFree(parser);
+      throw ex;
     }
 
   }
 
 				// Verify end of document.
   if (!XML_Parse(parser, buf, 0, true)) {
-    XML_ParserFree(parser);
-    throw sg_io_exception(XML_ErrorString(XML_GetErrorCode(parser)),
+    sg_io_exception ex (XML_ErrorString(XML_GetErrorCode(parser)),
 			  sg_location(path,
 				      XML_GetCurrentLineNumber(parser),
 				      XML_GetCurrentColumnNumber(parser)),
 			  "SimGear XML Parser");
+    XML_ParserFree(parser);
+    throw ex;
   }
 
   XML_ParserFree(parser);
@@ -273,4 +277,28 @@ readXML (const string &path, XMLVisitor &visitor)
   input.close();
 }
 
+void
+readXML (const char *buf, const int size, XMLVisitor &visitor)
+{
+  XML_Parser parser = XML_ParserCreate(0);
+  XML_SetUserData(parser, &visitor);
+  XML_SetElementHandler(parser, start_element, end_element);
+  XML_SetCharacterDataHandler(parser, character_data);
+  XML_SetProcessingInstructionHandler(parser, processing_instruction);
+
+  visitor.startXML();
+
+  if (!XML_Parse(parser, buf, size, false)) {
+      sg_io_exception ex (XML_ErrorString(XML_GetErrorCode(parser)),
+			    sg_location("In-memory XML buffer",
+					XML_GetCurrentLineNumber(parser),
+					XML_GetCurrentColumnNumber(parser)),
+			    "SimGear XML Parser");
+      XML_ParserFree(parser);
+      throw ex;
+  }
+
+  XML_ParserFree(parser);
+}
+
 // end of easyxml.cxx
